/*
  unicap
  Copyright (C) 2004  Arne Caspari

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __V4L2_H__
#define __V4L2_H__

#include <pthread.h>
#include <semaphore.h>


#define FOURCC(a,b,c,d) (unsigned int)((((unsigned int)d)<<24)+(((unsigned int)c)<<16)+(((unsigned int)b)<<8)+a)

#define MAX_V4L2_FORMATS      128
#define V4L2_MAX_VIDEO_INPUTS 32
#define V4L2_MAX_VIDEO_NORMS  32

enum _cpi_v4l2_io_method_enum
{
   CPI_V4L2_IO_METHOD_UNDECIDED = 0,
   CPI_V4L2_IO_METHOD_USERPOINTER, 
   CPI_V4L2_IO_METHOD_MMAP, 
   CPI_V4L2_IO_METHOD_READWRITE,
};

typedef enum _cpi_v4l2_io_method_enum cpi_v4l2_io_method_enum_t;

struct _cpi_v4l2_buffer
{
      void *start;
      size_t length;
};

struct _v4l2_handle
{
      char device[512];
      int fd;

      unicap_format_t *unicap_formats;
      int format_count;
      unicap_format_t current_format;
		
      unicap_property_t *unicap_properties;
      __u32 *control_ids;
      int property_count;
      int video_in_count;
      char *video_inputs[V4L2_MAX_VIDEO_INPUTS];
      char *video_norms[V4L2_MAX_VIDEO_NORMS];

      cpi_v4l2_io_method_enum_t io_method;
      int buffer_count;
      struct _cpi_v4l2_buffer *buffers; // for method mmap
      int *free_buffers;
	
      struct _unicap_queue *in_queue;
      int in_queue_lock;
      struct _unicap_queue *out_queue;
      int out_queue_lock;

      int capture_running;
      volatile int quit_capture_thread;
      volatile int dqindex;
      volatile int qindex;
      pthread_t capture_thread;
      
      sem_t sema;

      unicap_event_callback_t event_callback;
      unicap_handle_t unicap_handle;
};

typedef struct _v4l2_handle *v4l2_handle_t;

#endif//__V4L2_H__
