/*
** draw.c
** 
** Made by (Arne Caspari)
** Login   <arne@arne-laptop>
** 
** Started on  Mon Sep 18 18:14:12 2006 Arne Caspari
** Last update Sun Jan  7 18:52:33 2007 Arne Caspari
*/

#include "ucil.h"

#include <math.h>

#include "unicap.h"

#define FOURCC(a,b,c,d) (unsigned int)((((unsigned int)a))+(((unsigned int)b)<<8)+(((unsigned int)c)<<16)+(((unsigned int)d)<<24))
#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)<(b))?(b):(a))
#define CLIP(a,l,u) ((a)<(l)?(l):(((a)>(u)) ? (u) : (a) ) )	 

static __inline__ 
void draw_hline( unicap_data_buffer_t *data_buffer, ucil_color_t *color, unsigned int fill, int y, int x1, int x2 );


void ucil_convert_color( ucil_color_t *dest, ucil_color_t *src )
{
   int r,g,b,a,y,u,v;
   r=g=b=a=y=u=v = 0;

   switch( src->colorspace )
   {
      case UCIL_COLORSPACE_Y8:
	 r = g = b = y = src->y8.y;
	 u = v = 128;
	 a = 255;
	 break;
	 
      case UCIL_COLORSPACE_RGB24:
	 r = src->rgb24.r;
	 g = src->rgb24.g;
	 b = src->rgb24.b;
	 y = ( 0.2990 * (float)r + 0.5870 * (float)g + 0.1140 * (float)b );
	 u = (-0.1687 * (float)r - 0.3313 * (float)g + 0.5000 * (float)b + 128.0 );
	 v = ( 0.5000 * (float)r - 0.4187 * (float)g - 0.0813 * (float)b + 128.0);
	 break;
	 
      case UCIL_COLORSPACE_RGB32:
	 r = src->rgb32.r;
	 g = src->rgb32.g;
	 b = src->rgb32.b;
	 a = src->rgb32.a;
	 y = ( 0.2990 * (float)r + 0.5870 * (float)g + 0.1140 * (float)b );
	 u = (-0.1687 * (float)r - 0.3313 * (float)g + 0.5000 * (float)b + 128.0 );
	 v = ( 0.5000 * (float)r - 0.4187 * (float)g - 0.0813 * (float)b + 128.0);
	 break;
	 
      case UCIL_COLORSPACE_YUV:
	 y = src->yuv.y;
	 u = src->yuv.u;
	 v = src->yuv.v;
	 a = 255;
	 r = y - 0.0009267 * ( u - 128.0 ) + 1.4016868 * ( v - 128.0 );
	 g = y - 0.3436954 * ( u - 128.0 ) - 0.7141690 * ( v - 128.0 );
	 b = y + 1.7721604 * ( u - 128.0 ) + 0.0009902 * ( v - 128.0 );
	 r = CLIP(r,0,255);
	 g = CLIP(g,0,255);
	 b = CLIP(b,0,255);
	 break;

      default:
	 break;
   }

   switch( dest->colorspace )
   {
      case UCIL_COLORSPACE_Y8:
	 dest->y8.y = y;
	 break;
	 
      case UCIL_COLORSPACE_RGB24:
	 dest->rgb24.r = r;
	 dest->rgb24.g = g;
	 dest->rgb24.b = b;
	 break;
	 
      case UCIL_COLORSPACE_RGB32:
	 dest->rgb32.r = r;
	 dest->rgb32.g = g;
	 dest->rgb32.b = b;
	 dest->rgb32.a = a;
	 break;
	 
      case UCIL_COLORSPACE_YUV:
	 dest->yuv.y = (char)y;
	 dest->yuv.u = (char)u;
	 dest->yuv.v = (char)v;
	 break;
   }
}

void ucil_get_pixel( unicap_data_buffer_t *data_buffer, ucil_color_t *color, int x, int y )
{
   int width = data_buffer->format.size.width;
   switch( data_buffer->format.fourcc )
   {
      case FOURCC( 'Y', '8', '0', '0' ):
      case FOURCC( 'G', 'R', 'E', 'Y' ):
      {
	 color->colorspace = UCIL_COLORSPACE_Y8;
	 color->y8.y = data_buffer->data[ y * width + x ];
      }
      break;      
      
      case FOURCC( 'R', 'G', 'B', ' ' ):
      case FOURCC( 'R', 'G', 'B', '3' ):
      {
	 int o = y * width * 3 + x * 3;
	 color->colorspace = UCIL_COLORSPACE_RGB24;
	 color->rgb24.r = data_buffer->data[ o ];
	 color->rgb24.g = data_buffer->data[ o+1 ];
	 color->rgb24.b = data_buffer->data[ o+2 ];
      }
      break;
      
      case FOURCC( 'R', 'G', 'B', 'A' ):
      case FOURCC( 'R', 'G', 'B', '4' ):
      {
	 int o = y * width * 4 + x * 4;
	 color->colorspace = UCIL_COLORSPACE_RGB32;
	 color->rgb32.r = data_buffer->data[ o ];
	 color->rgb32.g = data_buffer->data[ o+1 ];
	 color->rgb32.b = data_buffer->data[ o+2 ];
	 color->rgb32.a = data_buffer->data[ o+3 ];
      }
      break;
      
      case FOURCC( 'U', 'Y', 'V', 'Y' ):
      {
	 int o = y * width * 2 + (x & ~1) * 2;
	 int p = ( x & 1 ) * 2;

	 color->colorspace = UCIL_COLORSPACE_YUV;
	 
	 color->yuv.y = data_buffer->data[ o + p + 1 ];
	 color->yuv.u = data_buffer->data[ o ];
	 color->yuv.v = data_buffer->data[ o + 2 ];
      }
      break;

   }
}
   



void ucil_set_pixel( unicap_data_buffer_t *data_buffer, ucil_color_t *color, int x, int y )
{
   int width = data_buffer->format.size.width;

   switch( data_buffer->format.fourcc )
   {
      case UCIL_FOURCC( 'Y', '8', '0', '0' ):
      case UCIL_FOURCC( 'G', 'R', 'E', 'Y' ):
      {
	 data_buffer->data[ y * width + x ] = color->y8.y;
      }
      break;      
      
      case UCIL_FOURCC( 'R', 'G', 'B', ' ' ):
      case UCIL_FOURCC( 'R', 'G', 'B', '3' ):
      {
	 int o = y * width * 3 + x * 3;
	 if( color->colorspace == UCIL_COLORSPACE_RGB24 )
	 {
	    data_buffer->data[ o ] = color->rgb24.r;
	    data_buffer->data[ o+1 ] = color->rgb24.g;
	    data_buffer->data[ o+2 ] = color->rgb24.b;
	 }
	 else
	 {
	    data_buffer->data[ o ] = MIN( (int)data_buffer->data[ o ] * color->rgb32.a + (int)color->rgb32.r, 255 );
	    data_buffer->data[ o+1 ] = MIN( (int)data_buffer->data[ o+1 ] * color->rgb32.a + (int)color->rgb32.g, 255 );
	    data_buffer->data[ o+2 ] = MIN( (int)data_buffer->data[ o+2 ] * color->rgb32.a + (int)color->rgb32.b, 255 );
	 }
      }
      break;
      
      case UCIL_FOURCC( 'R', 'G', 'B', 'A' ):
      case UCIL_FOURCC( 'R', 'G', 'B', '4' ):
      {
	 int o = y * width * 4 + x * 4;
	 if( color->colorspace == UCIL_COLORSPACE_RGB24 )
	 {
	    data_buffer->data[ o ] = color->rgb24.r;
	    data_buffer->data[ o+1 ] = color->rgb24.g;
	    data_buffer->data[ o+2 ] = color->rgb24.b;
	 }
	 else
	 {
	    data_buffer->data[ o ] = MIN( (int)data_buffer->data[ o ] * color->rgb32.a + (int)color->rgb32.r, 255 );
	    data_buffer->data[ o+1 ] = MIN( (int)data_buffer->data[ o+1 ] * color->rgb32.a + (int)color->rgb32.g, 255 );
	    data_buffer->data[ o+2 ] = MIN( (int)data_buffer->data[ o+2 ] * color->rgb32.a + (int)color->rgb32.b, 255 );
	    data_buffer->data[ o+3 ] = color->rgb32.a;
	 }
      }
      break;
      
      case UCIL_FOURCC( 'U', 'Y', 'V', 'Y' ):
      {
	 int o = y * width * 2 + (x & ~1) * 2;
	 int p = ( x & 1 ) * 2;
	 
	 data_buffer->data[ o + p + 1 ] = color->yuv.y;
	 data_buffer->data[ o ] = color->yuv.u;
	 data_buffer->data[ o + 2 ] = color->yuv.v;
	 
      }
      break;

   }
}

void ucil_set_pixel_alpha( unicap_data_buffer_t *data_buffer, ucil_color_t *color, int alpha, int x, int y )
{
   int width = data_buffer->format.size.width;

   switch( data_buffer->format.fourcc )
   {
      case UCIL_FOURCC( 'Y', '8', '0', '0' ):
      case UCIL_FOURCC( 'G', 'R', 'E', 'Y' ):
      {
	 data_buffer->data[ y * width + x ] = ( ( data_buffer->data[ y * width + x ] * 255 - alpha ) + ( color->y8.y * alpha ) ) / 256;
      }
      break;      
      
      case UCIL_FOURCC( 'R', 'G', 'B', ' ' ):
      case UCIL_FOURCC( 'R', 'G', 'B', '3' ):
      {
	 int o = y * width * 3 + x * 3;
	 data_buffer->data[ o ] = ((data_buffer->data[ o ] * (255-alpha) ) + ( color->rgb24.r * alpha ) ) / 256;
	 data_buffer->data[ o+1 ] = ((data_buffer->data[ o+1 ] * (255-alpha) ) + ( color->rgb24.g * alpha ) ) / 256;
	 data_buffer->data[ o+2 ] = ((data_buffer->data[ o+2 ] * (255-alpha) ) + ( color->rgb24.b * alpha ) ) / 256;
      }
      break;
      
      case UCIL_FOURCC( 'R', 'G', 'B', 'A' ):
      case UCIL_FOURCC( 'R', 'G', 'B', '4' ):
      {
	 int o = y * width * 4 + x * 4;
	 data_buffer->data[ o ] = ((data_buffer->data[ o ] * (255-alpha) ) + ( color->rgb24.r * alpha ) ) / 256;
	 data_buffer->data[ o+1 ] = ((data_buffer->data[ o+1 ] * (255-alpha) ) + ( color->rgb24.g * alpha ) ) / 256;
	 data_buffer->data[ o+2 ] = ((data_buffer->data[ o+2 ] * (255-alpha) ) + ( color->rgb24.b * alpha ) ) / 256;
      }
      break;
      
      case UCIL_FOURCC( 'U', 'Y', 'V', 'Y' ):
      {
	 int o = y * width * 2 + (x & ~1) * 2;
	 int p = ( x & 1 ) * 2;
	 
	 data_buffer->data[ o + p + 1 ] = ( ( data_buffer->data[ o + p + 1 ] * ( 255 - alpha ) ) + ( color->yuv.y * alpha ) ) / 256;
	 data_buffer->data[ o ] = ( ( data_buffer->data[ o ] * ( 255 - alpha ) ) + ( color->yuv.u * alpha ) ) / 256;
	 data_buffer->data[ o + 2 ] = ( ( data_buffer->data[ o + 2 ] * ( 255 - alpha ) ) + ( color->yuv.v * alpha ) ) / 256;
      }
      break;

   }
}

void ucil_draw_line( unicap_data_buffer_t *data_buffer, ucil_color_t *color, 
		     int x0, int y0, 
		     int x1, int y1 )
{
   int steep = abs(y1-y0)>abs(x1-x0);
   int deltax;
   int deltay;
   int error;
   int ystep;
   int x,y;
   
   if( steep )
   {
      int tmp;
      
      tmp = x0;
      x0 = y0;
      y0 = tmp;
      
      tmp = x1;
      x1 = y1;
      y1 = tmp;
   }
   
   if( x0 > x1 )
   {
      int tmp;
      
      tmp = x0;
      x0 = x1;
      x1 = tmp;
      tmp = y0;
      y0 = y1;
      y1 = tmp;
   }
   
   deltax = x1 - x0;
   deltay = abs(y1-y0);
   error = 0;
   y = y0;
   
   if( y0 < y1 )
   {
      ystep = 1;
   }
   else
   {
      ystep = -1;
   }
   
   for( x = x0; x < x1; x++ )
   {
      if( steep )
      {
	 ucil_set_pixel( data_buffer, color, y, x );
      }
      else
      {
	 ucil_set_pixel( data_buffer, color, x, y );
      }
      
      error = error + deltay;
      
      if( 2 * error >= deltax )
      {
	 y = y + ystep;
	 error = error - deltax;
      }
   }
}

static __inline__ 
void draw_hline( unicap_data_buffer_t *data_buffer, ucil_color_t *color, unsigned int fill, int y, int x1, int x2 )
{
   int o;
   int x;
   int bytespp;

   bytespp = data_buffer->format.bpp/8;

   o = y * data_buffer->format.size.width * bytespp;

   for( x = x1; x < ( ( x1 & ~0x3 ) + 4 ); x++ )
   {
      ucil_set_pixel( data_buffer, color, x, y );
   }
   for( x = x2 & ~0x3; x <= x2; x++ )
   {
      ucil_set_pixel( data_buffer, color, x, y );
   }
   for( x = ((x1&~0x3)+4); x < ((x2)&~03); x+=4/bytespp )
   {
      *(int*)(data_buffer->data + o + ( x * bytespp)) = fill;
   }
}


void ucil_draw_rect( unicap_data_buffer_t *data_buffer, ucil_color_t *color, int x1, int y1, int x2, int y2 )
{
   if( x1 > x2 )
   {
      int t;
      t = x2;
      x2 = x1;
      x1 = t;
   }
   if( y1 > y2 )
   {
      int t;
      t = y2;
      y2 = y1;
      y1 = t;
   }
   
   switch( color->colorspace )
   {
      case UCIL_COLORSPACE_Y8: 
      {
	 unsigned int fill;
	 int y;
	 for( y = y1; y <= y2; y++ )
	 {
	    fill = color->y8.y | ( color->y8.y << 8 ) | ( color->y8.y << 16 ) | ( color->y8.y << 24 );
	    draw_hline( data_buffer, color, fill, y, x1, x2 );
	 }
      }
      break;

      case UCIL_COLORSPACE_RGB24:
      {
	 int x,y;
	 for( y = y1; y < y2; y++ )
	 {
	    for( x = x1; x < x2; x++ )
	    {
	       ucil_set_pixel( data_buffer, color, x, y );
	    }
	 }
      }
      break;
      
      case UCIL_COLORSPACE_RGB32: 
      {
	 unsigned int fill;
	 int y;
	 for( y = y1; y <= y2; y++ )
	 {
	    fill = color->rgb32.r | ( color->rgb32.g << 8 ) | ( color->rgb32.b << 16 ) | ( color->rgb32.a << 24 );
	    draw_hline( data_buffer, color, fill, y, x1, x2 );
	 }
      }
      break;

      case UCIL_COLORSPACE_YUV:
      {
	 switch( data_buffer->format.fourcc )
	 {
	    case FOURCC('U','Y','V','Y'):
	    {
	       unsigned int fill;
	       int y;
	       for( y = y1; y <= y2; y++ )
	       {
		  fill = color->yuv.u + ( color->yuv.y << 8 ) + ( color->yuv.v << 16 ) + ( color->yuv.y << 24 );
		  draw_hline( data_buffer, color, fill, y, x1, x2 );		  
	       }
	    }
	 }
      }
      break;
   }   
}

void ucil_draw_box( unicap_data_buffer_t *data_buffer, ucil_color_t *color, int x1, int y1, int x2, int y2 )
{
   if( x1 > x2 )
   {
      int t;
      t = x2;
      x2 = x1;
      x1 = t;
   }
   if( y1 > y2 )
   {
      int t;
      t = y2;
      y2 = y1;
      y1 = t;
   }
   
   switch( color->colorspace )
   {
      case UCIL_COLORSPACE_Y8: 
      {
	 unsigned int fill;
	 int y;
	 fill = color->y8.y | ( color->y8.y << 8 ) | ( color->y8.y << 16 ) | ( color->y8.y << 24 );
	 draw_hline( data_buffer, color, fill, y1, x1, x2 );
	 draw_hline( data_buffer, color, fill, y2, x1, x2 );
	 for( y = y1; y < y2; y++ )
	 {
	    ucil_set_pixel( data_buffer, color, x1, y );
	    ucil_set_pixel( data_buffer, color, x2, y );
	 }
      }
      break;

      case UCIL_COLORSPACE_RGB24:
      {
	 int x,y;
	 for( y = y1; y < y2; y++ )
	 {
	    ucil_set_pixel( data_buffer, color, x1, y );
	    ucil_set_pixel( data_buffer, color, x2, y );
	 }
	 for( x = x1; x < x2; x++ )
	 {
	    ucil_set_pixel( data_buffer, color, x, y1 );
	    ucil_set_pixel( data_buffer, color, x, y2 );
	 }
      }
      break;
      
      case UCIL_COLORSPACE_RGB32: 
      {
	 unsigned int fill;
	 int y;
	 fill = color->rgb32.r | ( color->rgb32.g << 8 ) | ( color->rgb32.b << 16 ) | ( color->rgb32.a << 24 );
	 draw_hline( data_buffer, color, fill, y1, x1, x2 );
	 draw_hline( data_buffer, color, fill, y2, x1, x2 );
	 for( y = y1; y < y2; y++ )
	 {
	    ucil_set_pixel( data_buffer, color, x1, y );
	    ucil_set_pixel( data_buffer, color, x2, y );
	 }
      }
      break;

      case UCIL_COLORSPACE_YUV:
      {
	 switch( data_buffer->format.fourcc )
	 {
	    case FOURCC('U','Y','V','Y'):
	    {
	       unsigned int fill;
	       int y;
	       fill = color->yuv.u + ( color->yuv.y << 8 ) + ( color->yuv.v << 16 ) + ( color->yuv.y << 24 );
	       draw_hline( data_buffer, color, fill, y1, x1, x2 );
	       draw_hline( data_buffer, color, fill, y2, x1, x2 );
	       for( y = y1; y < y2; y++ )
	       {
		  ucil_set_pixel( data_buffer, color, x1, y );
		  ucil_set_pixel( data_buffer, color, x2, y );
	       }
	    }
	 }
      }
      break;
   }   
}

void ucil_draw_circle( unicap_data_buffer_t *dest, ucil_color_t *color, int cx, int cy, int r )
{
   double step = 1/(double)r;
   double i;

   for( i = 0; i < 2 * M_PI; i += step )
   {
      ucil_set_pixel( dest, color, cx + sin( i ) * r, cy + sin( i + M_PI_2 ) * r );
   }
}

