/*
** font.c
** 
** Made by (Arne Caspari)
** Login   <arne@arne-laptop>
** 
** Started on  Tue Dec 19 18:30:39 2006 Arne Caspari
** Last update Mon Jan  8 07:28:02 2007 Arne Caspari
*/

//#include "yuvfont.h"
#include "unicap.h"
#include "ucil.h"

#include <ft2build.h>
#include FT_FREETYPE_H

#define DEFAULT_FONT "/usr/share/fonts/truetype/freefont/FreeMono.ttf"

static void draw_bitmap( unicap_data_buffer_t *dest,
			 ucil_color_t *color,
			 int xpos, int ypos,
			 FT_Bitmap*  bitmap,
			 FT_Int      x,
			 FT_Int      y)
{
   FT_Int  i, j, p, q;
   FT_Int  x_max = x + bitmap->width;
   FT_Int  y_max = y + bitmap->rows;
   
   
   for ( i = x, p = 0; i < x_max; i++, p++ )
   {
      for ( j = y, q = 0; j < y_max; j++, q++ )
      {
	 if ( ( i < 0 ) || 
	      ( j < 0 ) ||
	      ( ( i + xpos ) >= dest->format.size.width ) || 
	      ( ( j + ypos ) >= dest->format.size.height ) )
	 {
	    continue;
	 }

	 if( bitmap->buffer[q * bitmap->width + p] )
	 {
	    ucil_set_pixel_alpha( dest, color, bitmap->buffer[q * bitmap->width + p], i + xpos, j + ypos );
	 }
      }
   }
}



void ucil_draw_text( unicap_data_buffer_t *dest, ucil_color_t *color, char *text, int x, int y, int size, const char *font )
{
   FT_Library library;
   FT_Face    face;
   FT_Error   error;
   
   FT_GlyphSlot  slot;
   FT_Vector     pen;                    /* untransformed origin  */


   int num_chars;
   int n;
   int xpos = x;
   int ypos = y - dest->format.size.height;
   

   if( font == NULL )
   {
      font = DEFAULT_FONT;
   }
   
   num_chars = strlen(text);
   
   error = FT_Init_FreeType( &library );              /* initialize library */
   /* error handling omitted */
   
   error = FT_New_Face( library, font, 0, &face ); /* create face object */
   /* error handling omitted */

   /* use 50pt at 100dpi */
   error = FT_Set_Char_Size( face, size * 64, 0,
			     100, 0 );                /* set character size */
   /* error handling omitted */
  slot = face->glyph;

  for( n = 0; n < num_chars; n++ )
  {
     error = FT_Load_Char( face, text[n], FT_LOAD_RENDER );
     if( error )
     {
	continue;
     }
     
     draw_bitmap( dest, 
		  color,
		  xpos, ypos, 
		  &slot->bitmap,
		  slot->bitmap_left,
		  dest->format.size.height - slot->bitmap_top );
     xpos += slot->advance.x / 64;
     ypos += slot->advance.y / 64;
  }
  
  FT_Done_Face( face );
  FT_Done_FreeType( library );  
}


