/*
** ucil.c
** 
** Made by (Arne Caspari)
** Login   <arne@arne-laptop>
** 
** Started on  Mon Oct 30 18:10:30 2006 Arne Caspari
** Last update Wed Jan  3 22:11:34 2007 Arne Caspari
*/

#include "ucil.h"
#include "yuvops.h"

#include <glib.h>

void ucil_blend_alpha( unicap_data_buffer_t *dest, unicap_data_buffer_t *bg, unicap_data_buffer_t *fg, int alpha )
{
   switch( bg->format.fourcc )
   {
      case UCIL_FOURCC( 'U', 'Y', 'V', 'Y' ):
      {
	 ucil_blend_uyvy_a( dest, bg, fg, alpha );
      }
      break;

      case UCIL_FOURCC( 'R', 'G', 'B', '3' ):
      {
	 ucil_blend_rgb24_a( dest, bg, fg, alpha );
      }
      break;

      case UCIL_FOURCC( 'R', 'G', 'B', '4' ):
      {
	 ucil_blend_rgb32_a( dest, bg, fg, alpha );
      }
      break;

      default:
      {
	 g_warning( "Operation not defined for color format\n" );
      }
      break;
   }
}

void ucil_fill( unicap_data_buffer_t *buffer, ucil_color_t *color )
{
   switch( buffer->format.fourcc )
   {
      case UCIL_FOURCC( 'U', 'Y', 'V', 'Y' ):
      {
	 ucil_fill_uyvy( buffer, color );
      }
      break;

      case UCIL_FOURCC( 'R', 'G', 'B', '3' ):
      {
	 ucil_fill_rgb24( buffer, color );
      }
      break;

      case UCIL_FOURCC( 'R', 'G', 'B', '4' ):
      {
	 ucil_fill_rgb32( buffer, color );
      }
      break;

      default:
      {
	 g_warning( "Operation not defined for color format\n" );
      }
      break;
   }
}
      
void ucil_convolution_mask( unicap_data_buffer_t *dest, unicap_data_buffer_t *src, ucil_convolution_mask_t *mask )
{
   switch( src->format.fourcc )
   {
      case UCIL_FOURCC( 'U', 'Y', 'V', 'Y' ):
      {
	 ucil_convolution_mask_uyvy( dest, src, mask );
      }
      break;
      
      default:
      {
	 g_warning( "Operation not defined for color format\n" );
      }
      break;
   }
}

ucil_convolution_mask_t *ucil_create_convolution_mask( unsigned char *array, int size, ucil_colorspace_t cs, int mode )
{
   ucil_convolution_mask_t *mask;
   
   mask = malloc( sizeof( ucil_convolution_mask_t ) );
   mask->mask = malloc( size * size * sizeof( ucil_color_t ) );
   mask->size = size;
   mask->colorspace = cs;

   switch( cs )
   {
      case UCIL_COLORSPACE_YUV:
      {
	 int i, j;
	 
	 for( i = 0; i < size; i++ )
	 {
	    for( j = 0; j < size; j++ )
	    {
	       mask->mask[i * size + j].colorspace = cs;
	       mask->mask[i * size + j].yuv.y = array[ i * size + j ];
	       mask->mask[i * size + j].yuv.u = 0;
	       mask->mask[i * size + j].yuv.v = 0;
	    }
	 }
      }
   }
   
   return mask;
}

ucil_colorspace_t ucil_get_colorspace_from_fourcc( unsigned int fourcc )
{
   ucil_colorspace_t cs = UCIL_COLORSPACE_UNKNOWN;
   
   switch( fourcc )
   {
      case UCIL_FOURCC( 'Y', '8', '0', '0' ):
      case UCIL_FOURCC( 'G', 'R', 'E', 'Y' ):
	 cs = UCIL_COLORSPACE_Y8;
	 break;
	 
      case UCIL_FOURCC( 'R', 'G', 'B', ' ' ):
      case UCIL_FOURCC( 'R', 'G', 'B', '3' ):
	 cs = UCIL_COLORSPACE_RGB24;
	 break;
	 
      case UCIL_FOURCC( 'R', 'G', 'B', 'A' ):
      case UCIL_FOURCC( 'R', 'G', 'B', '4' ):
	 cs = UCIL_COLORSPACE_RGB32;
	 break;
	 
      case UCIL_FOURCC( 'U', 'Y', 'V', 'Y' ):
	 cs = UCIL_COLORSPACE_YUV;
	 break;
	 
      default:
	 cs = UCIL_COLORSPACE_UNKNOWN;
	 break;
   }
   
   return cs;
}

