/*
** ucil.h
** 
** Made by Arne Caspari
** Login   <arne@arne-laptop>
** 
** Started on  Mon Sep 18 18:15:11 2006 Arne Caspari
** Last update Mon Jan  8 07:28:14 2007 Arne Caspari
*/

#ifndef   	UCIL_H_
# define   	UCIL_H_

#include "unicap.h"

#define UCIL_FOURCC(a,b,c,d) (unsigned int)((((unsigned int)d)<<24)+(((unsigned int)c)<<16)+(((unsigned int)b)<<8)+a)

enum ucil_colorspace
{
   UCIL_COLORSPACE_RGB24, 
   UCIL_COLORSPACE_RGB32, 
   UCIL_COLORSPACE_Y8, 
   UCIL_COLORSPACE_YUV, 
   
   UCIL_COLORSPACE_UNKNOWN = 0xfffff
};

typedef enum ucil_colorspace ucil_colorspace_t;

struct ucil_rgb24
{
      unsigned char r;
      unsigned char g;
      unsigned char b;
};

typedef struct ucil_rgb24 ucil_rgb24_t;

struct ucil_rgb32
{
      unsigned char r;
      unsigned char g;
      unsigned char b;
      unsigned char a;
};

typedef struct ucil_rgb32 ucil_rgb32_t;

struct ucil_yuv
{
      unsigned char y;
      unsigned char u;
      unsigned char v;
};

typedef struct ucil_yuv ucil_yuv_t;

struct ucil_y8
{
      unsigned char y;
};

typedef struct ucil_y8 ucil_y8_t;

struct ucil_color
{
      ucil_colorspace_t colorspace;
      
      union
      {
	    ucil_yuv_t yuv;
	    ucil_rgb24_t rgb24;
	    ucil_rgb32_t rgb32;
	    ucil_y8_t y8;
      };
};

typedef struct ucil_color ucil_color_t;

struct ucil_convolution_mask
{
      ucil_colorspace_t colorspace;
      
      ucil_color_t *mask;
      int size;
};

typedef struct ucil_convolution_mask ucil_convolution_mask_t;

struct ucil_text_object
{
      unsigned char *bitmap;
      int width;
      int height;
};

typedef struct ucil_text_object ucil_text_object_t;


/*
  Drawing functions
*/
void ucil_set_pixel( unicap_data_buffer_t *data_buffer, ucil_color_t *color, int x, int y );
void ucil_set_pixel_alpha( unicap_data_buffer_t *data_buffer, ucil_color_t *color, int alpha, int x, int y );
void ucil_draw_line( unicap_data_buffer_t *data_buffer, ucil_color_t *color, int x1, int y1, int x2, int y2 );
void ucil_draw_rect( unicap_data_buffer_t *data_buffer, ucil_color_t *color, int x1, int y1, int x2, int y2 );
void ucil_fill( unicap_data_buffer_t *data_buffer, ucil_color_t *color );
void ucil_draw_box( unicap_data_buffer_t *data_buffer, ucil_color_t *color, int x1, int y1, int x2, int y2 );
void ucil_draw_circle( unicap_data_buffer_t *dest, ucil_color_t *color, int cx, int cy, int r );
void ucil_draw_text( unicap_data_buffer_t *dest, ucil_color_t *color, char *text, int x, int y, int size, const char *font );


void ucil_get_pixel( unicap_data_buffer_t *data_buffer, ucil_color_t *color, int x, int y );

/*
  Colorspace transformation
 */
void ucil_convert_color( ucil_color_t *dest, ucil_color_t *src );
void ucil_convert_buffer( unicap_data_buffer_t *dest, unicap_data_buffer_t *src );


/*
  Buffer operations
 */
void ucil_blend_alpha( unicap_data_buffer_t *dest, unicap_data_buffer_t *bg, unicap_data_buffer_t *fg, int alpha );
void ucil_convolution_mask( unicap_data_buffer_t *dest, unicap_data_buffer_t *src, ucil_convolution_mask_t *mask );

ucil_convolution_mask_t *ucil_create_convolution_mask( unsigned char *array, int size, ucil_colorspace_t cs, int mode );

#endif 	    /* !UCIL_H_ */
