/*
** yuvops.c
** 
** Made by (Arne Caspari)
** Login   <arne@arne-laptop>
** 
** Started on  Mon Oct 30 07:20:28 2006 Arne Caspari
** Last update Wed Jan  3 18:31:42 2007 Arne Caspari
*/

#include "yuvops.h"
#include "unicap.h"
#include "ucil.h"

#include <sys/types.h>
#include <linux/types.h>



#define CLIP(v,l,h) (((v)<(l))?(l):(v)>(h)?(h):(v))


void ucil_blend_uyvy_a( unicap_data_buffer_t *dest, 
			unicap_data_buffer_t *img1, 
			unicap_data_buffer_t *img2, 
			int alpha )
{
   int offset = 0;
   int af = alpha;
   int ab = ( 1 << 16 ) - alpha;
   
   while( offset < img1->format.buffer_size )
   {
      __u8 y1,y2,u,v;
      
      int bu  = ( ( ((int)img1->data[offset])-127 ) * ab ) >> 16;
      int by1 = ( ((int)img1->data[offset+1]) * ab ) >> 16;
      int bv  = ( ( ((int)img1->data[offset+2])-127 ) * ab ) >> 16;
      int by2 = ( ((int)img1->data[offset+3]) * ab ) >> 16;

      int fu  = ( ( ((int)img2->data[offset])-127 ) * af ) >> 16;
      int fy1 = ( ((int)img2->data[offset+1]) * af ) >> 16;
      int fv  = ( ( ((int)img2->data[offset+2])-127 ) * af ) >> 16;
      int fy2 = ( ((int)img2->data[offset+3]) * af ) >> 16;
      
      dest->data[offset++]  = ( CLIP( ( bu + fu ), -127, 128 ) + 127 );
      dest->data[offset++] = CLIP( ( by1 + fy1 ), 0, 255 );
      dest->data[offset++]  = ( CLIP( ( bv + fv ), -127, 128 ) + 127 );
      dest->data[offset++] = CLIP( ( by2 + fy2 ), 0, 255 );      
   }
}

void ucil_fill_uyvy( unicap_data_buffer_t *buffer, 
		     ucil_color_t *color )
{
   int offset;

   __u32 word;
   
   word = ( color->yuv.y << 24 ) | (color->yuv.v << 16 ) | ( color->yuv.y << 8 ) | ( color->yuv.u );
   
   for( offset = 0; offset < ( buffer->format.buffer_size / 4 ); offset++ )
   {
      *(((__u32 *)buffer->data) + offset ) = word;
   }
}

void ucil_convolution_mask_uyvy( unicap_data_buffer_t *dest, 
				 unicap_data_buffer_t *src, 
				 ucil_convolution_mask_t *mask )
{
   int ix, iy;
   int mx, my;
   int mmoff;
   
   static int printed = 0;
   
   mmoff = ( ( (float)mask->size + 0.5 ) / 2 ) -1;

   for( iy = 1; iy < ( src->format.size.height - 1 ); iy++ )
   {
      for( ix = 1; ix < ( src->format.size.width - 1 ); ix++ )
      {
	 int val = 0;
	 int stride = src->format.size.width * 2;
	 int iyoff = iy * stride;
	 int ixoff = ix * 2;
	 
	 for( my = 0; my < mask->size; my++ )
	 {

	    for( mx = 0; mx < mask->size; mx++ )
	    {
	       int p, m;
	       int offset;
/* 	       if( !printed ) */
/* 	       { */
/* 		  printf( "%d ", mask->mask[ my * mask->size + mx ].yuv.y ); */
/* 	       } */
	       
	       offset = iyoff + my * stride + ixoff + mx * 2;
	       m = (signed char)mask->mask[ my * mask->size + mx ].yuv.y;
	       p = src->data[ offset + 1];	       
	       
	       val += ( p * m );

/* 	       if( !printed ) */
/* 	       { */
/* 		  printf( "[%d,%d] ", p,val ); */
/* 	       } */

	    }
/* 	    if( !printed ) */
/* 	    { */
/* 	       printf( "\n" ); */
/* 	    } */
	 }	 

	 dest->data[ iyoff + mmoff * stride + ixoff + mmoff * 2 ] = src->data[ iyoff + mmoff * stride + ixoff + mmoff * 2 ];
	 dest->data[ iyoff + mmoff * stride + ixoff + mmoff * 2 + 1] = CLIP( val, 0, 255 );
	 
	 printed = 1;
	 
      }
   }
}
