/* unicap
 *
 * Copyright (C) 2004 Arne Caspari ( arne_caspari@users.sourceforge.net )
 *
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gdk/gdk.h>

#include <string.h>

#include "config.h"

#include "cs_xfm.h"
#include "colorspace.h"

#if UNICAPGTK_DEBUG
#define DEBUG
#endif
#include "debug.h"


void cs_xfm_pixbuf_destroy_fn( guchar *pixels, gpointer data )
{
/* 	TRACE( "pixbuf destroy: %p\n", data ); */
   if( data )
   {
      g_free( data );
   }
}

GdkPixbuf *cs_xfm_new_pixbuf( unicap_data_buffer_t *image_buffer )
{
   GdkPixbuf *pixbuf;
   guchar *image_data;
   gint image_data_size;							  

   switch( image_buffer->format.fourcc )
   {
      case RGB3:
	 image_data_size = image_buffer->format.size.width *
	    image_buffer->format.size.height * 3;
	 image_data = image_buffer->data;
	 pixbuf = gdk_pixbuf_new_from_data( image_data,
					    GDK_COLORSPACE_RGB,
					    0, // alpha
					    8, // bps
					    image_buffer->format.size.width,
					    image_buffer->format.size.height,
					    image_buffer->format.size.width * 3,
					    cs_xfm_pixbuf_destroy_fn,
					    0 );
	 break;

      case RGB4:
	 image_data_size = image_buffer->format.size.width *
	    image_buffer->format.size.height * 4;
	 image_data = image_buffer->data;
	 pixbuf = gdk_pixbuf_new_from_data( image_data,
					    GDK_COLORSPACE_RGB,
					    1, // alpha
					    8, // bps
					    image_buffer->format.size.width,
					    image_buffer->format.size.height,
					    image_buffer->format.size.width * 4,
					    cs_xfm_pixbuf_destroy_fn,
					    0 );
	 break;
			
      default:
      {
	 xfm_info_t xfminfo;
	 
	 image_data_size = image_buffer->format.size.width *
	    image_buffer->format.size.height * 3;
			
	 unicapgtk_cs_get_xfminfo_from_fourcc( image_buffer->format.fourcc,
					       FOURCC( 'R', 'G', 'B', 0 ),
					       &xfminfo );
	 if( xfminfo.func )
	 {
	    image_data = (guchar *) g_malloc( image_data_size );
	    xfminfo.func( image_data,
			  image_buffer->data,
			  image_buffer->format.size.width,
			  image_buffer->format.size.height );

	    pixbuf = gdk_pixbuf_new_from_data( image_data,
					       GDK_COLORSPACE_RGB,
					       0, // alpha
					       8, // bps
					       image_buffer->format.size.width,
					       image_buffer->format.size.height,
					       image_buffer->format.size.width * 3,
					       cs_xfm_pixbuf_destroy_fn,
					       image_data );
	 }
	 else
	 {
	    image_data = (guchar *) g_malloc0( image_data_size );
	    
	    pixbuf = gdk_pixbuf_new_from_data( image_data,
					       GDK_COLORSPACE_RGB,
					       0, // alpha
					       8, // bps
					       image_buffer->format.size.width,
					       image_buffer->format.size.height,
					       image_buffer->format.size.width * 3,
					       cs_xfm_pixbuf_destroy_fn,
					       image_data );
	 }  
      }
      break;			
   }

   return pixbuf;
}

/*
 */
GdkPixbuf *cs_xfm_new_allocated_pixbuf( unicap_data_buffer_t *image_buffer )
{
   GdkPixbuf *pixbuf;
   guchar *image_data;
   gint image_data_size;
									  

   switch( image_buffer->format.fourcc )
   {
      case RGB3:
	 image_data_size = image_buffer->format.size.width * 
	    image_buffer->format.size.height * 3;
	 image_data = (guchar *) g_malloc( image_data_size );
	 memcpy( image_data, image_buffer->data, image_data_size );
	 pixbuf = gdk_pixbuf_new_from_data( image_data,
					    GDK_COLORSPACE_RGB,
					    0, // alpha
					    8, // bps
					    image_buffer->format.size.width,
					    image_buffer->format.size.height,
					    image_buffer->format.size.width * 3,
					    cs_xfm_pixbuf_destroy_fn,
					    image_data );
	 break;

      case RGB4:
	 image_data_size = image_buffer->format.size.width * 
	    image_buffer->format.size.height * 4;
	 image_data = (guchar *) g_malloc( image_data_size );
	 memcpy( image_data, image_buffer->data, image_data_size );
	 pixbuf = gdk_pixbuf_new_from_data( image_data,
					    GDK_COLORSPACE_RGB,
					    1, // alpha
					    8, // bps
					    image_buffer->format.size.width,
					    image_buffer->format.size.height,
					    image_buffer->format.size.width * 4,
					    cs_xfm_pixbuf_destroy_fn,
					    image_data );
	 break;
			
      default:
      {
	 xfm_info_t xfminfo;

	 image_data_size = image_buffer->format.size.width * 
	    image_buffer->format.size.height * 3;
	 image_data = (guchar *) g_malloc( image_data_size );
	 TRACE( "cs_xfm_new_pixbuf: UYVY\n" );
		 unicapgtk_cs_get_xfminfo_from_fourcc( image_buffer->format.fourcc, 
					       FOURCC( 'R', 'G', 'B', 0 ), 
					       &xfminfo );
	 if( xfminfo.func )
	 {
	    xfminfo.func( image_data,
			  image_buffer->data,
			  image_buffer->format.size.width, 
			  image_buffer->format.size.height );

	    pixbuf = gdk_pixbuf_new_from_data( image_data,
					       GDK_COLORSPACE_RGB,
					       0, // alpha
					       8, // bps
					       image_buffer->format.size.width,
					       image_buffer->format.size.height,
					       image_buffer->format.size.width * 3,
					       cs_xfm_pixbuf_destroy_fn,
					       image_data );
	 }
	 else
	 {
	    image_data_size = image_buffer->format.size.width * 
	       image_buffer->format.size.height * 3;
	    image_data = (guchar *) g_malloc0( image_data_size );
	    
	    pixbuf = gdk_pixbuf_new_from_data( image_data,
					       GDK_COLORSPACE_RGB,
					       0, // alpha
					       8, // bps
					       image_buffer->format.size.width,
					       image_buffer->format.size.height,
					       image_buffer->format.size.width * 3,
					       cs_xfm_pixbuf_destroy_fn,
					       image_data );
	 }  
      }
      
      break;
			
   }

   return pixbuf;
}
