/* unicap
 *
 * Copyright (C) 2004 Arne Caspari ( arne_caspari@users.sourceforge.net )
 *
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <string.h>

#include "cs_xfm.h"
#include "backend.h"
#include "config.h"

#if UNICAPGTK_DEBUG
#define DEBUG
#endif
#include <debug.h>

#include <sys/time.h>
#include <time.h>
#include <semaphore.h>

#define NUM_BUFFERS 2


struct backend_data
{
      GtkWidget *widget;
      guchar *image_data[ NUM_BUFFERS ];
      gint current_buffer;
      unicap_format_t format;
      GdkPixbuf *pixbuf;

      gint crop_x;
      gint crop_y;
      gint crop_w;
      gint crop_h;

      sem_t sema;
};

gint backend_get_merit()
{
   return BACKEND_MERIT_GTK;
}

gboolean backend_init( GtkWidget *widget, unicap_format_t *format, gpointer *_data, GError **err )
{
   struct backend_data *data;
   int i;
   
   data = g_new0( struct backend_data, 1 );

   g_memmove( &data->format, format, sizeof( unicap_format_t ) );

   for( i = 0; i < NUM_BUFFERS; i++ )
   {
      data->image_data[ i ] = g_malloc0( format->size.width * format->size.height * 3 );
      g_assert( data->image_data );
   }

   data->crop_x = 0;
   data->crop_y = 0;
   data->crop_w = format->size.width;
   data->crop_h = format->size.height;
   data->pixbuf = NULL;
 
   data->widget = widget;

   *_data = data;
   
   sem_init( &data->sema, 0, 1 );

   return TRUE;
}

void backend_destroy( gpointer _data )
{
   struct backend_data *data = _data;
   int i;
   
   for( i = 0; i < NUM_BUFFERS; i++ )
   {
      g_free( data->image_data[ i ] );
   }
   g_free( data );
}


void backend_update_image( gpointer _data, unicap_data_buffer_t *data_buffer, GError **err )
{
   struct backend_data *data = _data;
   int tmp;

   tmp = ( data->current_buffer + 1 ) % NUM_BUFFERS;

   if( !data_buffer )
   {
      g_warning( "update_image: data_buffer == NULL!\n" );
      return;
   }
   
   if( ( data_buffer->format.fourcc != data->format.fourcc ) || 
       ( data_buffer->format.size.width != data->format.size.width ) || 
       ( data_buffer->format.size.height != data->format.size.height ) ||
       ( data_buffer->format.bpp != data->format.bpp ) )
   {
      g_warning( "update_image: data_buffer format mismatch\n" );
      return;
   }
   
   cs_xfm_to_RGB24( data_buffer, data->image_data[ tmp ] );
}

void backend_display_image( gpointer _data )
{
   struct backend_data *data = _data;

   data->current_buffer = ( data->current_buffer + 1 ) % NUM_BUFFERS;
}


void backend_redraw( gpointer _data )
{
   struct backend_data *data = _data;

   if( data->image_data )
   {
      gdk_draw_rgb_image( data->widget->window, 
			  data->widget->style->fg_gc[GTK_STATE_NORMAL], 
			  data->crop_x, data->crop_y, 
			  data->crop_w, data->crop_h, 
			  GDK_RGB_DITHER_NONE, 
			  data->image_data[ data->current_buffer ], 
			  data->format.size.width * 3 );      
   }
   
}

void backend_expose_event( gpointer _data, GtkWidget *da, GdkEventExpose *event )
{
   backend_redraw( _data );
   
}

void backend_get_image_data( gpointer _data, unicap_data_buffer_t *data_buffer, int b )
{
   struct backend_data *data = _data;
   int tmp;

   tmp = ( data->current_buffer + 1 ) % NUM_BUFFERS;
   
   data_buffer->format.fourcc = RGB3;
   data_buffer->format.bpp = 24;
   data_buffer->format.size.width = data->format.size.width;
   data_buffer->format.size.height = data->format.size.height;
   data_buffer->format.buffer_size = data->format.size.width * data->format.size.height * 3;
   data_buffer->data = data->image_data[ tmp ];
   data_buffer->buffer_size = data_buffer->format.buffer_size;
}

void backend_lock( gpointer _data )
{
   struct backend_data *data = _data;
   sem_wait( &data->sema );
}

void backend_unlock( gpointer _data )
{
   struct backend_data *data = _data;
   sem_post( &data->sema );
}
