/* unicap
 *
 * Copyright (C) 2004 Arne Caspari ( arne_caspari@users.sourceforge.net )
 *
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gdk/gdk.h>

#include <string.h>

#include "cs_xfm.h"
#include "colorspace.h"


size_t cs_xfm_get_image_data_size( unicap_format_t *format )
{
   size_t size = 0;

   switch( format->fourcc )
   {
      case RGB3:
	 size = format->size.width * format->size.height * 3;
	 break;

      case RGB4:
	 size = format->size.width * format->size.height * 4;
	 break;
			
      default:
	 size = format->size.width * format->size.height * 3;
   }
   
   return size;
}

void cs_xfm_to_RGB24( unicap_data_buffer_t *data_buffer, guchar *dest )
{
   switch( data_buffer->format.fourcc )
   {
      case RGB3:
	 memcpy( dest, data_buffer->data, data_buffer->buffer_size );
	 break;
	 
/*       case RGB4: */
/* 	 g_warning( "No conversion method for video format!\n" ); */
/* 	 break; */
	 
      default:
      {
	 xfm_info_t xfminfo;
	 
	 unicapgtk_cs_get_xfminfo_from_fourcc( data_buffer->format.fourcc,
					       FOURCC( 'R', 'G', 'B', 0 ),
					       &xfminfo );
	 if( xfminfo.func )
	 {
	    xfminfo.func( dest,
			  data_buffer->data,
			  data_buffer->format.size.width,
			  data_buffer->format.size.height );
	 }
	 else
	 {
	    g_warning( "No conversion method for video format!\n" );
	 }
      }
   }   
}

void cs_xfm_to_RGB32( unicap_data_buffer_t *data_buffer, guchar *dest )
{
   switch( data_buffer->format.fourcc )
   {
      case RGB3:
	 memcpy( data_buffer->data, dest, data_buffer->buffer_size );
	 break;
	 
      case RGB4:
	 g_assert( "No conversion" == FALSE );
	 break;
	 
      default:
      {
	 xfm_info_t xfminfo;
	 
	 unicapgtk_cs_get_xfminfo_from_fourcc( data_buffer->format.fourcc,
					       FOURCC( 'R', 'G', 'B', 'A' ),
					       &xfminfo );
	 if( xfminfo.func )
	 {
	    xfminfo.func( dest,
			  data_buffer->data,
			  data_buffer->format.size.width,
			  data_buffer->format.size.height );
	 }
	 else
	 {
	    g_assert( "No conversion" == FALSE );
	 }
      }
   }   
}


GdkPixbuf *cs_xfm_new_pixbuf( unicap_data_buffer_t *image_buffer, guchar *image_data )
{
   GdkPixbuf *pixbuf;

   switch( image_buffer->format.fourcc )
   {
      case RGB3:
	 pixbuf = gdk_pixbuf_new_from_data( image_data,
					    GDK_COLORSPACE_RGB,
					    0, // alpha
					    8, // bps
					    image_buffer->format.size.width,
					    image_buffer->format.size.height,
					    image_buffer->format.size.width * 3,
					    NULL,
					    0 );
	 break;

      case RGB4:
	 pixbuf = gdk_pixbuf_new_from_data( image_data,
					    GDK_COLORSPACE_RGB,
					    1, // alpha
					    8, // bps
					    image_buffer->format.size.width,
					    image_buffer->format.size.height,
					    image_buffer->format.size.width * 4,
					    NULL,
					    0 );
	 break;
			
      default:
      {
	 xfm_info_t xfminfo;
	 
	 unicapgtk_cs_get_xfminfo_from_fourcc( image_buffer->format.fourcc,
					       FOURCC( 'R', 'G', 'B', 0 ),
					       &xfminfo );
	 if( xfminfo.func )
	 {
	    xfminfo.func( image_data,
			  image_buffer->data,
			  image_buffer->format.size.width,
			  image_buffer->format.size.height );

	    pixbuf = gdk_pixbuf_new_from_data( image_data,
					       GDK_COLORSPACE_RGB,
					       0, // alpha
					       8, // bps
					       image_buffer->format.size.width,
					       image_buffer->format.size.height,
					       image_buffer->format.size.width * 3,
					       NULL,
					       0 );
	 }
	 else
	 {
	    pixbuf = gdk_pixbuf_new_from_data( image_data,
					       GDK_COLORSPACE_RGB,
					       0, // alpha
					       8, // bps
					       image_buffer->format.size.width,
					       image_buffer->format.size.height,
					       image_buffer->format.size.width * 3,
					       NULL,
					       0 );
	 }  
      }
      break;			
   }

   return pixbuf;
}
